// Get Mars central meridian.
// Algorithm from 2001 RASC Observer's Handbook, p. 172,
// converted to use integer math only so it'll work in PalmOS
// as well as Unix.
// Copyright 2001 by Akkana Peck.

typedef struct ephem_s {
    long sec_offset;     // offset from June 14, a day after opposition
    long L1_10;          // L1 * 10
    long delta_100;      // delta * 100
} ephem_t;

ephem_t Ephem2001[] = {
    { -1382400, 3135, 891 },    // May 29
    { -691200, 2423, 886 },     // Jun 6
    { 0, 1714, 884 },           // Jun 14, just past opposition
    { 691200, 1007, 887 },      // Jun 22, just past closest approach
    { 1382400, 297, 893 },      // Jun 30
    { 2073600, 3183, 903 },     // Jul 8
    { 2764800, 2461, 912 },     // Jul 16
    { 3456000, 1732, 921 },     // Jul 24
    { 4147200, 995, 930 },      // Aug 1
    { 4838400, 251, 938 },      // Aug 9
    { 5529600, 3101, 944 },     // Aug 17
    // Add more later if this program turns out to be useful
};

int GetCentralMeridian(long seconds)
{
    // Find the appropriate ephem entry:
    ephem_t *ephem = 0;
    int i;
    long days;
    long hours;
    long L;

    for (i=1; i < (sizeof Ephem2001 / sizeof *Ephem2001); ++i)
    {
        if (Ephem2001[i].sec_offset > seconds) {
            ephem = &(Ephem2001[i-1]);
            break;
        }
    }
    if (!ephem) {
        ephem = &(Ephem2001[sizeof Ephem2001 / sizeof *Ephem2001]);
        printf("Oops!  Way past last ephemeris date\n");
    }

//#define DEBUG

#ifdef DEBUG
    printf("seconds: %ld\n", seconds);
    printf("Using ephem entry of %ld, %ld, %ld\n",
           ephem->sec_offset, ephem->L1_10, ephem->delta_100);
#endif
    seconds -= ephem->sec_offset;
    days = (int)(seconds / 24/60/60);
    hours = (seconds - days * 24*60*60) /60/60;   // Might need TZ adjustment
#ifdef DEBUG
    printf("Seconds: %ld, Days: %ld, hours: %ld\n", seconds, days, hours);
#endif

    L = ephem->L1_10 - (ephem->delta_100 * days) / 10;
    // Now we're *10 from final
    L += 146 * hours;
    L /= 10;

    while (L < 0) L += 360;
    while (L > 360) L -= 360;

    return (int)L;
}

